# -*- coding: utf-8 -*-

# Copyright (C) 2015 Avencall
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

import consul
import json
import os
import yaml

from xivo_auth.token import TokenPayload, Storage
from xivo.chain_map import ChainMap
from xivo.config_helper import parse_config_file


def main():
    config = _read_config()

    consul_client = consul.Consul(**config['consul'])
    storage = Storage(consul_client)
    token_updater = _TokenUpdater(storage)

    os.umask(027)

    token_updater.update(
        'xivo-agid',
        ['acl:dird'],
        _Rules().read('xivo/private/').build(),
    )


def _read_config():
    consul_config = parse_config_file('/var/lib/consul/default_xivo_consul_config.yml')
    consul_token = parse_config_file('/var/lib/consul/default_xivo_consul_token.yml')
    return ChainMap(consul_token, consul_config)


class _TokenUpdater(object):

    def __init__(self, storage):
        self._storage = storage

    def update(self, name, acls, rules):
        token = self._upsert_token(name, acls, rules)
        self._write_config_file(name, token)

    def _upsert_token(self, name, acls, rules):
        token_payload = TokenPayload(auth_id=name, acls=acls)
        token = self._storage.upsert_named_token(name, token_payload, rules)
        return token

    def _write_config_file(self, name, token):
        filename = '/var/lib/xivo-manage-tokens/{}-token.yml'.format(name)
        with open(filename, 'w') as fobj:
            yaml.safe_dump({'token': token.token}, fobj)


class _Rules(object):

    def __init__(self):
        self._rules = []

    def read(self, key):
        self._rules.append((key, 'read'))
        return self

    def write(self, key):
        self._rules.append((key, 'write'))
        return self

    def deny(self, key):
        self._rules.append((key, 'deny'))
        return self

    def build(self):
        return json.dumps({'key': {key: {'policy': policy} for key, policy in self._rules}})
